@@ -1,6 +1,5 @@
 import json
 import os
-import re
 from collections import OrderedDict
 
 import pandas as pd
@@ -10,30 +9,20 @@ from sim2bids.generate import subjects
 import sim2bids.templates.templates as temp
 
 # define naming conventions
-DEFAULT_TEMPLATE = '{}_{}.{}'
-COORD_TEMPLATE = '{}.{}'
+DEFAULT_TEMPLATE = '{}_desc-{}_{}.{}'
+COORD_TEMPLATE = 'desc-{}_{}.{}'
 
 # set to true if centres.txt (nodes and labels) are the same
 # for all files. In that case, store only one copy of the files
-# (json, tsv) in the main 'coord' folder, in the global scope
+# in the main 'coord' folder in the same scope as CHANGES.txt
 IGNORE_CENTRE = False
-IGNORE_AREAS = False
-IGNORE_CORTICAL = False
-IGNORE_HEMISPHERE = False
-IGNORE_NORMALS = False
 
 # location of coord files (nodes and labels) in the scope of
 # converted files. This information is used to supplement JSON
 # sidecars, specifically `CoordsRows` and `CoordsColumns`
 COORDS = None
 
-# set true if `times` are the same for all files. In that case,
-# store only one copy of the files (json, tsv) in the main 'coord'
-# folder, in the global scope
-IGNORE_TIMES = False
-
 NETWORK = []
-TIMES_TO_SKIP = []
 
 
 def save(sub: dict, folders: list, ses: str = None, name: str = None) -> None:
@@ -83,7 +72,7 @@ def save(sub: dict, folders: list, ses: str = None, name: str = None) -> None:
     ----------
     sub (dict):
         Dictionary containing information of one file only. For example,
-        {'name': 'centres', 'fname': 'centres.txt', 'sid': '1', 'desc': 'default',
+        {'name': 'centres', 'fname': 'centres.txt', 'sid': 'sub-01', 'desc': 'default',
         'sep': '\t', 'path': PATH_TO_FILE, 'ext': 'txt'}
     folders (list):
         List of folders corresponding to whether input files have single- or
@@ -103,14 +92,11 @@ def save(sub: dict, folders: list, ses: str = None, name: str = None) -> None:
             'vars', 'stimuli', 'noise', 'spike', 'raster', 'ts', 'event', 'emp'     # Timeseries (ts)
             'fc'                                                                    # Spatial (spatial)
     """
-    global IGNORE_CENTRE, IGNORE_CORTICAL, IGNORE_AREAS, IGNORE_NORMALS, IGNORE_HEMISPHERE
-
-    # if name == 'centres' and sub['name'] not in ['nodes', 'labels']:
-    #     name = 'coord'
+    global IGNORE_CENTRE
 
     # check if centres should be ignored. If so, immediately break
     # the function. Otherwise, continue iteration.
-    if IGNORE_CENTRE is True and name == 'centres' and os.path.exists(os.path.join(app.OUTPUT, 'coord', 'nodes.txt')):
+    if IGNORE_CENTRE and name == 'centres':
         return
 
     # read file contents
@@ -125,12 +111,7 @@ def save(sub: dict, folders: list, ses: str = None, name: str = None) -> None:
             folder = folders[2]
 
         # get description for weights or distances
-        if 'scthran' in sub['name'].lower():
-            desc = temp.file_desc['weights'].format('thresholded ')
-        elif 'scnot' in sub['name'].lower():
-            desc = temp.file_desc['weights'].format('')
-        else:
-            desc = temp.file_desc['distances']
+        desc = temp.file_desc['weights'] if 'weights' in sub['name'] else temp.file_desc['distances']
 
         if 'content' in sub.keys():
             save_files(sub, folder, sub['content'], ftype='wd')
@@ -143,10 +124,8 @@ def save(sub: dict, folders: list, ses: str = None, name: str = None) -> None:
         save_centres(sub, file, ses, folders)
 
     # get folder location for spatial
-    elif name in ['spatial', 'fc', 'map']:
-        fname = subjects.accepted(sub['name'], True)
-        desc = temp.file_desc['spatial_map'] if 'map' in fname \
-               else temp.file_desc[subjects.accepted(fname, return_accepted=True)].format(sub['sid'])
+    elif name in ['spatial', 'fc']:
+        desc = temp.file_desc['spatial']
 
         if ses is None:
             folder = folders[2]
@@ -160,25 +139,19 @@ def save(sub: dict, folders: list, ses: str = None, name: str = None) -> None:
             save_files(sub, folder, file, type='default', ftype='spatial', desc=desc)
 
     # get folder location for time series
-    elif name in ['ts']:
-        desc = temp.file_desc['ts'].format(sub['sid']) if name == 'ts' else temp.file_desc['bold'].format(sub['sid'].replace('sub-', ''))
+    elif name in ['ts', 'bold']:
+        desc = temp.file_desc['ts'] if name == 'ts' else temp.file_desc['bold'].format(sub['sid'].replace('sub-', ''))
         save_files(sub, folders[-1], file, type='default', ftype='ts', desc=desc)
 
-    elif name == 'times':
-        traverse_times(sub, folders, ses)
-
     # get folder location for coordinates
     elif name == 'coord':
         # check nodes
-        if ('nodes' in sub['name'] or 'labels' in sub['name']) and not IGNORE_CENTRE:
+        if 'nodes' in sub['name'] and not IGNORE_CENTRE:
             if 'content' in sub.keys():
                 save_centres(sub, sub['content'], ses, folders, centre_name='nodes')
             else:
                 save_centres(sub, file, ses, folders, centre_name='nodes')
         else:
-            # set naming convention needed for conversion
-            type_ = 'default'
-
             # set appropriate output path depending on session and subject types
             if ses is not None:
                 folder = folders[4]
@@ -187,94 +160,27 @@ def save(sub: dict, folders: list, ses: str = None, name: str = None) -> None:
             else:
                 folder = os.path.join(app.OUTPUT, 'coord')
 
-            # check contents of areas
-            if 'areas' in sub['path']:
-                if IGNORE_AREAS is True:
-                    return
-                IGNORE_AREAS = check_file('areas', sub, folder, file)
-
-            # check contents of cortical
-            elif 'cortical' in sub['path']:
-                if IGNORE_CORTICAL is True:
-                    return
-                IGNORE_CORTICAL = check_file('cortical', sub, folder, file)
-
-            # check contents of cortical
-            elif 'normals' in sub['path']:
-                if IGNORE_NORMALS is True:
-                    return
-                IGNORE_NORMALS = check_file('normals', sub, folder, file)
-
-            # check contents of cortical
-            elif 'hemisphere' in sub['path']:
-                if IGNORE_HEMISPHERE is True:
-                    return
-                IGNORE_HEMISPHERE = check_file('hemisphere', sub, folder, file)
+            if 'content' in sub.keys():
+                save_files(sub, folder, file, type='default', centres=True)
             else:
-                accepted = subjects.get_name(sub['name'], True)
-
-                if app.SESSIONS:
-                    save_files(sub, folder, file, type='default', desc=temp.file_desc[accepted])
-                # elif IGNORE_CENTRE and 'ses' not in folder and app.SESSIONS is None:
-                elif IGNORE_CENTRE and not app.SESSIONS:
-                    save_files(sub, folder, file, type='other', desc=temp.file_desc[accepted])
-                else:
-                    accepted = subjects.get_name(sub['name'])
-
-                    if IGNORE_CENTRE and 'sub' in folder:
-                        type_ = 'default'
+                # save conversion results
+                if 'centres' in sub['fname']:
+                    if IGNORE_CENTRE or not app.MULTI_INPUT:
+                        save_files(sub, folder, file, type='other', centres=True, desc=temp.centres['single'])
                     else:
-                        type_ = 'coord'
-
-                    save_files(sub, folder, file, type=type_, desc=temp.file_desc[accepted])
-
-
-            #
-            # if 'content' in sub.keys():
-            #     save_files(sub, folder, file, type='default', centres=True)
-            # else:
-            #     if sub['name'] in ['nodes', 'labels']:
-            #         save_centres(sub, file, ses, folders)
-            #
-            #     # save conversion results
-            #     if 'centres' in sub['fname']:
-            #         if IGNORE_CENTRE or not app.MULTI_INPUT:
-            #             save_files(sub, folder, file, type='other', centres=True, desc=temp.centres['single'])
-            #         else:
-            #             save_files(sub, folder, file, type='default', centres=True, desc=temp.centres['multi-unique'])
-            #     else:
-            #         accepted = subjects.get_name(sub['name'], True)
-            #
-            #         if app.SESSIONS:
-            #             save_files(sub, folder, file, type='default', desc=temp.file_desc[accepted])
-            #         # elif IGNORE_CENTRE and 'ses' not in folder and app.SESSIONS is None:
-            #         elif IGNORE_CENTRE and not app.SESSIONS:
-            #             save_files(sub, folder, file, type='other', desc=temp.file_desc[accepted])
-            #         else:
-            #             accepted = subjects.get_name(sub['name'])
-            #
-            #             if IGNORE_CENTRE and 'sub' in folder:
-            #                 type_ = 'default'
-            #             else:
-            #                 type_ = 'coord'
-            #
-            #             save_files(sub, folder, file, type=type_, desc=temp.file_desc[accepted])
-
-
-def check_file(name, sub, folder, file):
-    if check_coords(name):
-        save_files(sub, os.path.join(app.OUTPUT, 'coord'), file, type='other', desc=temp.file_desc[name])
-        return True
-
-    save_files(sub, folder, file, type='default', desc=temp.file_desc[name])
-    return False
+                        save_files(sub, folder, file, type='default', centres=True, desc=temp.centres['multi-unique'])
+                else:
+                    if IGNORE_CENTRE or not app.MULTI_INPUT:
+                        save_files(sub, folder, file, type='other', desc=temp.file_desc[sub['name']])
+                    else:
+                        save_files(sub, folder, file, type='default', desc=temp.file_desc[sub['name']])
 
 
 def save_centres(sub, file, ses, folders, centre_name='centres'):
     global IGNORE_CENTRE
 
     # check if all centres are of the same content
-    if check_coords(name=centre_name):
+    if check_centres(name=centre_name):
         # ignore preceding centres files
         IGNORE_CENTRE = True
 
@@ -288,15 +194,12 @@ def save_centres(sub, file, ses, folders, centre_name='centres'):
         desc = temp.centres['multi-unique'] if app.MULTI_INPUT else temp.centres['single']
 
         # set appropriate output path depending on session and subject types
-        if len(folders) > 1:
-            if ses is not None:
-                folder = folders[4]
-            elif app.MULTI_INPUT and ses is None:
-                folder = folders[3]
-            else:
-                folder = os.path.join(app.OUTPUT, 'coord')
+        if ses is not None:
+            folder = folders[4]
+        elif app.MULTI_INPUT and ses is None:
+            folder = folders[3]
         else:
-            folder = folders[0]
+            folder = os.path.join(app.OUTPUT, 'coord')
 
         # save conversion results
         if IGNORE_CENTRE:
@@ -321,7 +224,7 @@ def save_files(sub: dict, folder: str, content, type: str = 'default', centres:
     ----------
     sub (dict):
         Dictionary containing information of one file only. For example,
-        {'name': 'centres', 'fname': 'centres.txt', 'sid': '1', 'desc': 'default',
+        {'name': 'centres', 'fname': 'centres.txt', 'sid': 'sub-01', 'desc': 'default',
         'sep': '\t', 'path': PATH_TO_FILE, 'ext': 'txt'}
     folder (str):
         Folder where to store output conversions for the specific file. For example,
@@ -354,14 +257,11 @@ def save_files(sub: dict, folder: str, content, type: str = 'default', centres:
     """
     global COORDS
 
-    name = sub['name'].replace('.txt', '')
-
     if type == 'default':
-        json_file = os.path.join(folder, DEFAULT_TEMPLATE.format(sub['sid'], name, 'json'))
+        json_file = os.path.join(folder, DEFAULT_TEMPLATE.format(sub['sid'], sub['desc'], sub['name'], 'json'))
         tsv_file = json_file.replace('json', 'tsv')
-
     else:
-        json_file = os.path.join(folder, COORD_TEMPLATE.format(name, 'json'))
+        json_file = os.path.join(folder, COORD_TEMPLATE.format(sub['desc'], sub['name'], 'json'))
         tsv_file = json_file.replace('json', 'tsv')
 
     # Save 'centres.txt' as 'nodes.txt' and 'labels.txt'. This will require breaking the
@@ -393,15 +293,17 @@ def save_files(sub: dict, folder: str, content, type: str = 'default', centres:
         to_tsv(nodes.replace('json', 'tsv'), content.iloc[:, 1:])
     else:
         if ftype == 'coord':
-            to_json(json_file, shape=content.shape, key='coord', desc=desc)
-            to_tsv(tsv_file, content)
+            if sub['name'] == 'bold_times':
+                desc = temp.file_desc['bold_times'].format(sub['sid'])
+            to_json(json_file.lower(), shape=content.shape, key='coord', desc=desc)
+            to_tsv(tsv_file.lower(), content)
         else:
             # otherwise, save files as usual
-            to_json(json_file, shape=content.shape, key=ftype, desc=desc)
-            to_tsv(tsv_file, content)
+            to_json(json_file.lower(), shape=content.shape, key=ftype, desc=desc)
+            to_tsv(tsv_file.lower(), content)
 
 
-def check_coords(name='centres'):
+def check_centres(name='centres'):
     """
     This function checks all centres files in the input folder and
     decides whether they have the same content or not. The logic of
@@ -415,24 +317,31 @@ def check_coords(name='centres'):
     """
 
     # get all centres files
-    files = get_specific(name)
+    centres = get_specific(name)
 
-    if len(files) == 1:
-        return True
+    # get the first element
+    file = open_file(centres[0], subjects.find_separator(centres[0]))
 
-    if files:
-        # get the first element
-        f1 = open_file(files[0], subjects.find_separator(files[0]))
-        f2 = open_file(files[1], subjects.find_separator(files[1]))
+    # define set literal
+    same = {}
 
-        if f1.equals(f2):
+    # iterate over centres
+    for centre in centres[1:]:
+        # append to the set literal whether the contents of the first element
+        # are the same with the rest
+        same.update(file == open_file(centre, subjects.find_separator(centre)))
+
+    # check if set literal contains only one element
+    if len(same) == 1:
+        if bool(same):
+            # if files are the same, return True
             return True
+        # False otherwise
         return False
-
     return False
 
 
-def get_specific(filetype: str, constraint: str = None) -> list:
+def get_specific(filetype: str) -> list:
     """
     Get all files that correspond to the filetype. For example,
     if filetype is equal to "areas", this function will return
@@ -453,7 +362,6 @@ def get_specific(filetype: str, constraint: str = None) -> list:
     Returns
     -------
         Returns a list of all files that end with the specified filetype.
-        :param constraint:
     """
 
     # create emtpy list to store appropriate files
@@ -463,11 +371,8 @@ def get_specific(filetype: str, constraint: str = None) -> list:
     for file in app.ALL_FILES:
         # check if the keyword is present
         if filetype in file:
-            if constraint is not None and constraint in file:
-                content.append(file)
-            if constraint is None:
-                # if yes, append the value
-                content.append(file)
+            # if yes, append the value
+            content.append(file)
 
     # return the list of newly-found files
     return content
@@ -502,53 +407,6 @@ def open_file(path: str, sep: str):
         pass
 
 
-def traverse_times(sub, folders, ses):
-    global TIMES_TO_SKIP
-
-    # get description
-    desc = temp.file_desc['times'] if 'bold' not in sub['name'] else temp.file_desc['bold_times']
-
-    # check if times are similar
-    for times in app.TIMES:
-        for rhythm in ['alpha', 'delta', 'beta', 'gamma', 'theta']:
-            # check if the file needs to be skipped
-            if f'{rhythm}_{times}' in TIMES_TO_SKIP:
-                return
-
-            results = get_specific(times, rhythm)
-
-            if len(results) > 0:
-                open_df = lambda x: pd.read_csv(x, header=None, sep='\t')
-                first = open_df(results[0])
-
-                if len(results) > 1:
-                    for result in results[1:]:
-                        if first.equals(open_df(result)):
-                            similar = True
-                            TIMES_TO_SKIP.append(f'{rhythm}_{times}')
-
-                            # save in global folder
-                            sub['name'] = f'{rhythm}-{times}'
-                            save_files(sub, os.path.join(app.OUTPUT, 'coord'), first, 'coord', desc=desc)
-                        else:
-                            # set appropriate output path depending on session and subject types
-                            if ses is not None:
-                                folder = folders[4]
-                            elif app.MULTI_INPUT and ses is None:
-                                folder = folders[3]
-                            else:
-                                folder = os.path.join(app.OUTPUT, 'coord')
-
-                            save_files(sub, folder, first, 'default', desc=desc)
-                            save_files(sub, folder, open_df(result), 'default', desc=desc)
-                else:
-                    similar = True
-                    TIMES_TO_SKIP.append(f'{rhythm}_{times}')
-                    # save in global folder
-                    sub['name'] = f'{rhythm}-{times.replace(".txt", "")}'
-                    save_files(sub, os.path.join(app.OUTPUT, 'coord'), first, 'coord', desc=desc)
-
-
 def open_text(path, sep):
     """
 
@@ -567,7 +425,7 @@ def open_text(path, sep):
         return pd.read_csv(path, header=None, index_col=None)
 
     try:
-        f = pd.read_csv(path, sep=sep, header=None, index_col=None)
+        f = pd.read_csv(path, sep=sep, header=None, index_col=False)
     except pd.errors.EmptyDataError:
         return ''
     except ValueError:
@@ -628,91 +486,35 @@ def to_json(path, shape, desc, key, **kwargs):
     -------
 
     """
-    global NETWORK
 
-    if key not in ['param', 'eq', 'code']:
-        out = OrderedDict({x: '' for x in temp.required})
-    else:
-        out = OrderedDict()
+    inp = temp.required
+    out = OrderedDict({x: '' for x in inp})
+
+    params = {'ModelEq': f'../eq/desc-{app.DESC}_eq.xml',
+              'ModelParam': f'../param/desc-{app.DESC}_param.xml',
+              'SourceCode': f'../code/desc-{app.DESC}_code.py',
+              'SoftwareVersion': app.SoftwareVersion,
+              'SoftwareRepository': app.SoftwareRepository,
+              'SourceCodeVersion': app.SoftwareVersion,
+              'SoftwareName': app.SoftwareName,
+              'Network': NETWORK
+              }
 
     if key != 'wd':
         struct = temp.struct[key]
         out.update({x: '' for x in struct['required']})
 
-    # ===========================================================
-    #                     TAKE CARE OF COORDS
-    # ===========================================================
-    coord = None
+    for k in ['ModelEq', 'ModelParam', 'SourceCode', 'SoftwareVersion', 'SourceCodeVersion',
+              'SoftwareRepository', 'SoftwareName', 'Network']:
+        if k in out.keys():
+            out[k] = params[k]
 
-    if 'CoordsRows' in out.keys() or key in ['wd', 'ts', 'spatial']:
-        if app.SESSIONS and IGNORE_CENTRE:
-            coord = ['../../../coord/nodes.json', '../../../coord/labels.json']
-        elif app.SESSIONS and not IGNORE_CENTRE:
-            coord = ['../coord/nodes.json', '../coord/labels.json']
-        elif not app.SESSIONS and IGNORE_CENTRE:
-            coord = ['../../coord/nodes.json', '../../coord/labels.json']
-        else:
-            coord = ['../coord/nodes.json', '../coord/labels.json']
+    if 'Units' in out.keys() and key != 'coord':
+        out['Units'] = 'ms'
 
-    if key != 'wd':
-
-        # ===========================================================
-        #                     TAKE CARE OF EQUATIONS
-        # ===========================================================
-
-        if 'ModelEq' in out.keys() or 'ModelEq' in temp.struct[key]['recommend']:
-            if 'param' in path or 'code' in path:
-                out['ModelEq'] = '../eq/eq.xml'
-            else:
-                if app.SESSIONS:
-                    out['ModelEq'] = '../../../eq/eq.xml'
-                else:
-                    out['ModelEq'] = '../../eq/eq.xml'
-
-        # ===========================================================
-        #                     TAKE CARE OF PARAMETERS
-        # ===========================================================
-
-        if 'ModelParam' in out.keys() or 'ModelParam' in temp.struct[key]['recommend']:
-            if 'eq' in path or 'code' in path:
-                out['ModelParam'] = '../param/param.xml'
-            else:
-                if app.SESSIONS:
-                    out['ModelParam'] = '../../../param/param.xml'
-                else:
-                    out['ModelParam'] = '../../param/param.xml'
-
-        params = {
-            'SourceCode': f'/tvb-framework-{app.SoftwareVersion}' if app.SoftwareCode == 'MISSING' else app.SoftwareCode,
-            'SoftwareVersion': app.SoftwareVersion if app.SoftwareVersion else None,
-            'SoftwareRepository': app.SoftwareRepository if app.SoftwareRepository else None,
-            'SourceCodeVersion': app.SoftwareVersion if app.SoftwareVersion else None,
-            'SoftwareName': app.SoftwareName if app.SoftwareName else None,
-            'Network': NETWORK if NETWORK else None
-        }
-
-        if 'eq' not in path or 'param' not in path or 'code' not in path:
-            sub = re.findall(r'sub-[0-9]+', path)
-            network = None
-
-            if sub:
-                network = [f'../net/{sub[0]}_{app.DESC}_weights.json', f'../net/{sub[0]}_{app.DESC}_distances.json']
-
-            if network:
-                params['Network'] = network
-
-        if kwargs:
-            params += kwargs
-
-        for k in params.keys():
-            if k in temp.struct[key]['required'] or k in temp.struct[key]['recommend']:
-                out[k] = params[k]
-
-    if 'Units' in out.keys():
-        if key == 'wd' or key == 'coord':
-            out['Units'] = ''
-        else:
-            out['Units'] = 'ms'
+    # point coord files to nodes/labels TSV files instead
+    coord = COORDS if key != 'coord' else [COORDS[0].replace('json', 'tsv'), COORDS[1].replace('json', 'tsv')] \
+        if COORDS is not None else COORDS
 
     with open(path, 'w') as file:
-        json.dump(temp.populate_dict(out, shape=shape, desc=desc, coords=coord), file)
+        json.dump(temp.populate_dict(out, shape=shape, desc=desc, coords=coord, **kwargs), file)
